@@ -1,4 +1,7 @@
 
+#' @import jmvcore
+NULL
+
 # aggregate from R 3.3 for use in R 3.2
 
 #' @importFrom stats aggregate.data.frame
@@ -78,3 +81,51 @@ aggregate <- function (x, by, FUN, ..., simplify = TRUE, drop = TRUE)
     row.names(y) <- NULL
     y
 }
+
+tapply = function (X, INDEX, FUN = NULL, ..., simplify = TRUE, drop = TRUE)
+{
+    FUN <- if (!is.null(FUN))
+        match.fun(FUN)
+    if (!is.list(INDEX))
+        INDEX <- list(INDEX)
+    INDEX <- lapply(INDEX, as.factor)
+    nI <- length(INDEX)
+    if (!nI)
+        stop("'INDEX' is of length zero")
+    if (!all(lengths(INDEX) == length(X)))
+        stop("arguments must have same length")
+    namelist <- lapply(INDEX, levels)
+    extent <- lengths(namelist, use.names = FALSE)
+    cumextent <- cumprod(extent)
+    if (cumextent[nI] > .Machine$integer.max)
+        stop("total number of levels >= 2^31")
+    storage.mode(cumextent) <- "integer"
+    ngroup <- cumextent[nI]
+    group <- as.integer(INDEX[[1L]])
+    if (nI > 1L)
+        for (i in 2L:nI) group <- group + cumextent[i - 1L] *
+        (as.integer(INDEX[[i]]) - 1L)
+    if (is.null(FUN))
+        return(group)
+    levels(group) <- as.character(seq_len(ngroup))
+    class(group) <- "factor"
+    ans <- split(X, group)
+    names(ans) <- NULL
+    if (drop) {
+        index <- as.logical(lengths(ans))
+    } else {
+        index <- rep(TRUE, length(ans))
+    }
+    ans <- lapply(X = ans[index], FUN = FUN)
+    if (simplify && all(lengths(ans) == 1L)) {
+        ansmat <- array(dim = extent, dimnames = namelist)
+        ans <- unlist(ans, recursive = FALSE)
+    } else {
+        ansmat <- array(vector("list", prod(extent)), dim = extent,
+                        dimnames = namelist)
+    }
+    if (length(ans)) {
+        ansmat[index] <- ans
+    }
+    ansmat
+}
